@@ -1,24 +1,25 @@
 package libpod
 
 import (
+	"bytes"
 	"fmt"
 	"io/ioutil"
 	"net"
 	"os"
-	"path/filepath"
 	"time"
 
 	"github.com/containernetworking/cni/pkg/types"
 	cnitypes "github.com/containernetworking/cni/pkg/types/current"
-	"github.com/containers/image/v4/manifest"
-	"github.com/containers/libpod/libpod/define"
-	"github.com/containers/libpod/libpod/lock"
-	"github.com/containers/libpod/pkg/namespaces"
-	"github.com/containers/libpod/pkg/rootless"
+	"github.com/containers/common/pkg/secrets"
+	"github.com/containers/image/v5/manifest"
+	"github.com/containers/podman/v3/libpod/define"
+	"github.com/containers/podman/v3/libpod/lock"
+	"github.com/containers/podman/v3/pkg/rootless"
 	"github.com/containers/storage"
 	"github.com/cri-o/ocicni/pkg/ocicni"
 	spec "github.com/opencontainers/runtime-spec/specs-go"
 	"github.com/pkg/errors"
+	"github.com/sirupsen/logrus"
 )
 
 // CgroupfsDefaultCgroupParent is the cgroup parent for CGroupFS in libpod
@@ -32,15 +33,6 @@ const SystemdDefaultCgroupParent = "machine.slice"
 // manager in libpod when running as rootless
 const SystemdDefaultRootlessCgroupParent = "user.slice"
 
-// JournaldLogging is the string conmon expects to specify journald logging
-const JournaldLogging = "journald"
-
-// KubernetesLogging is the string conmon expects when specifying to use the kubernetes logging format
-const KubernetesLogging = "k8s-file"
-
-// JSONLogging is the string conmon expects when specifying to use the json logging format
-const JSONLogging = "json-file"
-
 // DefaultWaitInterval is the default interval between container status checks
 // while waiting.
 const DefaultWaitInterval = 250 * time.Millisecond
@@ -92,20 +84,6 @@ func (ns LinuxNS) String() string {
 	}
 }
 
-// Valid restart policy types.
-const (
-	// RestartPolicyNone indicates that no restart policy has been requested
-	// by a container.
-	RestartPolicyNone = ""
-	// RestartPolicyNo is identical in function to RestartPolicyNone.
-	RestartPolicyNo = "no"
-	// RestartPolicyAlways unconditionally restarts the container.
-	RestartPolicyAlways = "always"
-	// RestartPolicyOnFailure restarts the container on non-0 exit code,
-	// with an optional maximum number of retries.
-	RestartPolicyOnFailure = "on-failure"
-)
-
 // Container is a single OCI container.
 // All operations on a Container that access state must begin with a call to
 // syncContainer().
@@ -134,13 +112,22 @@ type Container struct {
 	rootlessSlirpSyncR *os.File
 	rootlessSlirpSyncW *os.File
 
+	rootlessPortSyncR *os.File
+	rootlessPortSyncW *os.File
+
 	// A restored container should have the same IP address as before
 	// being checkpointed. If requestedIP is set it will be used instead
 	// of config.StaticIP.
 	requestedIP net.IP
+	// A restored container should have the same MAC address as before
+	// being checkpointed. If requestedMAC is set it will be used instead
+	// of config.StaticMAC.
+	requestedMAC net.HardwareAddr
 
 	// This is true if a container is restored from a checkpoint.
 	restoreFromCheckpoint bool
+
+	slirp4netnsSubnet *net.IPNet
 }
 
 // ContainerState contains the current state of the container
@@ -173,9 +160,13 @@ type ContainerState struct {
 	PID int `json:"pid,omitempty"`
 	// ConmonPID is the PID of the container's conmon
 	ConmonPID int `json:"conmonPid,omitempty"`
-	// ExecSessions contains active exec sessions for container
-	// Exec session ID is mapped to PID of exec process
-	ExecSessions map[string]*ExecSession `json:"execSessions,omitempty"`
+	// ExecSessions contains all exec sessions that are associated with this
+	// container.
+	ExecSessions map[string]*ExecSession `json:"newExecSessions,omitempty"`
+	// LegacyExecSessions are legacy exec sessions from older versions of
+	// Podman.
+	// These are DEPRECATED and will be removed in a future release.
+	LegacyExecSessions map[string]*legacyExecSession `json:"execSessions,omitempty"`
 	// NetworkStatus contains the configuration results for all networks
 	// the pod is attached to. Only populated if we created a network
 	// namespace for the container, and the network namespace is currently
@@ -202,206 +193,14 @@ type ContainerState struct {
 	// and not delegated to the OCI runtime.
 	ExtensionStageHooks map[string][]spec.Hook `json:"extensionStageHooks,omitempty"`
 
+	// NetInterfaceDescriptions describe the relationship between a CNI
+	// network and an interface names
+	NetInterfaceDescriptions ContainerNetworkDescriptions `json:"networkDescriptions,omitempty"`
+
 	// containerPlatformState holds platform-specific container state.
 	containerPlatformState
 }
 
-// ExecSession contains information on an active exec session
-type ExecSession struct {
-	ID      string   `json:"id"`
-	Command []string `json:"command"`
-	PID     int      `json:"pid"`
-}
-
-// ContainerConfig contains all information that was used to create the
-// container. It may not be changed once created.
-// It is stored, read-only, on disk
-type ContainerConfig struct {
-	Spec *spec.Spec `json:"spec"`
-	ID   string     `json:"id"`
-	Name string     `json:"name"`
-	// Full ID of the pood the container belongs to
-	Pod string `json:"pod,omitempty"`
-	// Namespace the container is in
-	Namespace string `json:"namespace,omitempty"`
-	// ID of this container's lock
-	LockID uint32 `json:"lockID"`
-
-	// TODO consider breaking these subsections up into smaller structs
-
-	// UID/GID mappings used by the storage
-	IDMappings storage.IDMappingOptions `json:"idMappingsOptions,omitempty"`
-
-	// Information on the image used for the root filesystem/
-	RootfsImageID   string `json:"rootfsImageID,omitempty"`
-	RootfsImageName string `json:"rootfsImageName,omitempty"`
-	// Rootfs to use for the container, this conflicts with RootfsImageID
-	Rootfs string `json:"rootfs,omitempty"`
-	// Whether to mount volumes specified in the image.
-	ImageVolumes bool `json:"imageVolumes"`
-	// Src path to be mounted on /dev/shm in container.
-	ShmDir string `json:"ShmDir,omitempty"`
-	// Size of the container's SHM.
-	ShmSize int64 `json:"shmSize"`
-	// Static directory for container content that will persist across
-	// reboot.
-	StaticDir string `json:"staticDir"`
-	// Mounts list contains all additional mounts into the container rootfs.
-	// These include the SHM mount.
-	// These must be unmounted before the container's rootfs is unmounted.
-	Mounts []string `json:"mounts,omitempty"`
-	// NamedVolumes lists the named volumes to mount into the container.
-	NamedVolumes []*ContainerNamedVolume `json:"namedVolumes,omitempty"`
-
-	// Security Config
-
-	// Whether the container is privileged
-	Privileged bool `json:"privileged"`
-	// SELinux process label for container
-	ProcessLabel string `json:"ProcessLabel,omitempty"`
-	// SELinux mount label for root filesystem
-	MountLabel string `json:"MountLabel,omitempty"`
-	// LabelOpts are options passed in by the user to setup SELinux labels
-	LabelOpts []string `json:"labelopts,omitempty"`
-	// User and group to use in the container
-	// Can be specified by name or UID/GID
-	User string `json:"user,omitempty"`
-	// Additional groups to add
-	Groups []string `json:"groups,omitempty"`
-
-	// Namespace Config
-	// IDs of container to share namespaces with
-	// NetNsCtr conflicts with the CreateNetNS bool
-	// These containers are considered dependencies of the given container
-	// They must be started before the given container is started
-	IPCNsCtr    string `json:"ipcNsCtr,omitempty"`
-	MountNsCtr  string `json:"mountNsCtr,omitempty"`
-	NetNsCtr    string `json:"netNsCtr,omitempty"`
-	PIDNsCtr    string `json:"pidNsCtr,omitempty"`
-	UserNsCtr   string `json:"userNsCtr,omitempty"`
-	UTSNsCtr    string `json:"utsNsCtr,omitempty"`
-	CgroupNsCtr string `json:"cgroupNsCtr,omitempty"`
-
-	// IDs of dependency containers.
-	// These containers must be started before this container is started.
-	Dependencies []string
-
-	// Network Config
-
-	// CreateNetNS indicates that libpod should create and configure a new
-	// network namespace for the container.
-	// This cannot be set if NetNsCtr is also set.
-	CreateNetNS bool `json:"createNetNS"`
-	// StaticIP is a static IP to request for the container.
-	// This cannot be set unless CreateNetNS is set.
-	// If not set, the container will be dynamically assigned an IP by CNI.
-	StaticIP net.IP `json:"staticIP"`
-	// PortMappings are the ports forwarded to the container's network
-	// namespace
-	// These are not used unless CreateNetNS is true
-	PortMappings []ocicni.PortMapping `json:"portMappings,omitempty"`
-	// UseImageResolvConf indicates that resolv.conf should not be
-	// bind-mounted inside the container.
-	// Conflicts with DNSServer, DNSSearch, DNSOption.
-	UseImageResolvConf bool
-	// DNS servers to use in container resolv.conf
-	// Will override servers in host resolv if set
-	DNSServer []net.IP `json:"dnsServer,omitempty"`
-	// DNS Search domains to use in container resolv.conf
-	// Will override search domains in host resolv if set
-	DNSSearch []string `json:"dnsSearch,omitempty"`
-	// DNS options to be set in container resolv.conf
-	// With override options in host resolv if set
-	DNSOption []string `json:"dnsOption,omitempty"`
-	// UseImageHosts indicates that /etc/hosts should not be
-	// bind-mounted inside the container.
-	// Conflicts with HostAdd.
-	UseImageHosts bool
-	// Hosts to add in container
-	// Will be appended to host's host file
-	HostAdd []string `json:"hostsAdd,omitempty"`
-	// Network names (CNI) to add container to. Empty to use default network.
-	Networks []string `json:"networks,omitempty"`
-	// Network mode specified for the default network.
-	NetMode namespaces.NetworkMode `json:"networkMode,omitempty"`
-
-	// Image Config
-
-	// UserVolumes contains user-added volume mounts in the container.
-	// These will not be added to the container's spec, as it is assumed
-	// they are already present in the spec given to Libpod. Instead, it is
-	// used when committing containers to generate the VOLUMES field of the
-	// image that is created, and for triggering some OCI hooks which do not
-	// fire unless user-added volume mounts are present.
-	UserVolumes []string `json:"userVolumes,omitempty"`
-	// Entrypoint is the container's entrypoint.
-	// It is not used in spec generation, but will be used when the
-	// container is committed to populate the entrypoint of the new image.
-	Entrypoint []string `json:"entrypoint,omitempty"`
-	// Command is the container's command.
-	// It is not used in spec generation, but will be used when the
-	// container is committed to populate the command of the new image.
-	Command []string `json:"command,omitempty"`
-
-	// Misc Options
-
-	// Whether to keep container STDIN open
-	Stdin bool `json:"stdin,omitempty"`
-	// Labels is a set of key-value pairs providing additional information
-	// about a container
-	Labels map[string]string `json:"labels,omitempty"`
-	// StopSignal is the signal that will be used to stop the container
-	StopSignal uint `json:"stopSignal,omitempty"`
-	// StopTimeout is the signal that will be used to stop the container
-	StopTimeout uint `json:"stopTimeout,omitempty"`
-	// Time container was created
-	CreatedTime time.Time `json:"createdTime"`
-	// NoCgroups indicates that the container will not create CGroups. It is
-	// incompatible with CgroupParent.
-	NoCgroups bool `json:"noCgroups,omitempty"`
-	// Cgroup parent of the container
-	CgroupParent string `json:"cgroupParent"`
-	// LogPath log location
-	LogPath string `json:"logPath"`
-	// LogDriver driver for logs
-	LogDriver string `json:"logDriver"`
-	// File containing the conmon PID
-	ConmonPidFile string `json:"conmonPidFile,omitempty"`
-	// RestartPolicy indicates what action the container will take upon
-	// exiting naturally.
-	// Allowed options are "no" (take no action), "on-failure" (restart on
-	// non-zero exit code, up an a maximum of RestartRetries times),
-	// and "always" (always restart the container on any exit code).
-	// The empty string is treated as the default ("no")
-	RestartPolicy string `json:"restart_policy,omitempty"`
-	// RestartRetries indicates the number of attempts that will be made to
-	// restart the container. Used only if RestartPolicy is set to
-	// "on-failure".
-	RestartRetries uint `json:"restart_retries,omitempty"`
-	// TODO log options for log drivers
-
-	// PostConfigureNetNS needed when a user namespace is created by an OCI runtime
-	// if the network namespace is created before the user namespace it will be
-	// owned by the wrong user namespace.
-	PostConfigureNetNS bool `json:"postConfigureNetNS"`
-
-	// OCIRuntime used to create the container
-	OCIRuntime string `json:"runtime,omitempty"`
-
-	// ExitCommand is the container's exit command.
-	// This Command will be executed when the container exits
-	ExitCommand []string `json:"exitCommand,omitempty"`
-	// IsInfra is a bool indicating whether this container is an infra container used for
-	// sharing kernel namespaces in a pod
-	IsInfra bool `json:"pause"`
-
-	// Systemd tells libpod to setup the container in systemd mode
-	Systemd bool `json:"systemd"`
-
-	// HealthCheckConfig has the health check command and related timings
-	HealthCheckConfig *manifest.Schema2HealthConfig `json:"healthcheck"`
-}
-
 // ContainerNamedVolume is a named volume that will be mounted into the
 // container. Each named volume is a libpod Volume present in the state.
 type ContainerNamedVolume struct {
@@ -414,6 +213,34 @@ type ContainerNamedVolume struct {
 	Options []string `json:"options,omitempty"`
 }
 
+// ContainerOverlayVolume is a overlay volume that will be mounted into the
+// container. Each volume is a libpod Volume present in the state.
+type ContainerOverlayVolume struct {
+	// Destination is the absolute path where the mount will be placed in the container.
+	Dest string `json:"dest"`
+	// Source specifies the source path of the mount.
+	Source string `json:"source,omitempty"`
+	// Options holds overlay volume options.
+	Options []string `json:"options,omitempty"`
+}
+
+// ContainerImageVolume is a volume based on a container image.  The container
+// image is first mounted on the host and is then bind-mounted into the
+// container.
+type ContainerImageVolume struct {
+	// Source is the source of the image volume.  The image can be referred
+	// to by name and by ID.
+	Source string `json:"source"`
+	// Dest is the absolute path of the mount in the container.
+	Dest string `json:"dest"`
+	// ReadWrite sets the volume writable.
+	ReadWrite bool `json:"rw"`
+}
+
+// ContainerNetworkDescriptions describes the relationship between the CNI
+// network and the ethN where N is an integer
+type ContainerNetworkDescriptions map[string]int
+
 // Config accessors
 // Unlocked
 
@@ -427,6 +254,11 @@ func (c *Container) Config() *ContainerConfig {
 	return returnConfig
 }
 
+// Runtime returns the container's Runtime.
+func (c *Container) Runtime() *Runtime {
+	return c.runtime
+}
+
 // Spec returns the container's OCI runtime spec
 // The spec returned is the one used to create the container. The running
 // spec may differ slightly as mounts are added based on the image
@@ -454,11 +286,9 @@ func (c *Container) specFromState() (*spec.Spec, error) {
 		if err := json.Unmarshal(content, &returnSpec); err != nil {
 			return nil, errors.Wrapf(err, "error unmarshalling container config")
 		}
-	} else {
+	} else if !os.IsNotExist(err) {
 		// ignore when the file does not exist
-		if !os.IsNotExist(err) {
-			return nil, errors.Wrapf(err, "error opening container config")
-		}
+		return nil, errors.Wrapf(err, "error opening container config")
 	}
 
 	return returnSpec, nil
@@ -486,15 +316,15 @@ func (c *Container) Namespace() string {
 	return c.config.Namespace
 }
 
-// Image returns the ID and name of the image used as the container's rootfs
+// Image returns the ID and name of the image used as the container's rootfs.
 func (c *Container) Image() (string, string) {
 	return c.config.RootfsImageID, c.config.RootfsImageName
 }
 
-// ImageVolumes returns whether the container is configured to create
-// persistent volumes requested by the image
-func (c *Container) ImageVolumes() bool {
-	return c.config.ImageVolumes
+// RawImageName returns the unprocessed and not-normalized user-specified image
+// name.
+func (c *Container) RawImageName() string {
+	return c.config.RawImageName
 }
 
 // ShmDir returns the sources path to be mounted on /dev/shm in container
@@ -543,6 +373,11 @@ func (c *Container) MountLabel() string {
 	return c.config.MountLabel
 }
 
+// Systemd returns whether the container will be running in systemd mode
+func (c *Container) Systemd() bool {
+	return c.config.Systemd
+}
+
 // User returns the user who the container is run as
 func (c *Container) User() string {
 	return c.config.User
@@ -710,6 +545,11 @@ func (c *Container) LogPath() string {
 	return c.config.LogPath
 }
 
+// LogTag returns the tag to the container's log file
+func (c *Container) LogTag() string {
+	return c.config.LogTag
+}
+
 // RestartPolicy returns the container's restart policy.
 func (c *Container) RestartPolicy() string {
 	return c.config.RestartPolicy
@@ -748,7 +588,10 @@ func (c *Container) Hostname() string {
 
 // WorkingDir returns the containers working dir
 func (c *Container) WorkingDir() string {
-	return c.config.Spec.Process.Cwd
+	if c.config.Spec.Process != nil {
+		return c.config.Spec.Process.Cwd
+	}
+	return "/"
 }
 
 // State Accessors
@@ -916,13 +759,13 @@ func (c *Container) ExecSession(id string) (*ExecSession, error) {
 
 	session, ok := c.state.ExecSessions[id]
 	if !ok {
-		return nil, errors.Wrapf(define.ErrNoSuchCtr, "no exec session with ID %s found in container %s", id, c.ID())
+		return nil, errors.Wrapf(define.ErrNoSuchExecSession, "no exec session with ID %s found in container %s", id, c.ID())
 	}
 
 	returnSession := new(ExecSession)
-	returnSession.ID = session.ID
-	returnSession.Command = session.Command
-	returnSession.PID = session.PID
+	if err := JSONDeepCopy(session, returnSession); err != nil {
+		return nil, errors.Wrapf(err, "error copying contents of container %s exec session %s", c.ID(), session.ID())
+	}
 
 	return returnSession, nil
 }
@@ -1045,6 +888,12 @@ func (c *Container) NamespacePath(linuxNS LinuxNS) (string, error) { //nolint:in
 		}
 	}
 
+	return c.namespacePath(linuxNS)
+}
+
+// namespacePath returns the path of one of the container's namespaces
+// If the container is not running, an error will be returned
+func (c *Container) namespacePath(linuxNS LinuxNS) (string, error) { //nolint:interfacer
 	if c.state.State != define.ContainerStateRunning && c.state.State != define.ContainerStatePaused {
 		return "", errors.Wrapf(define.ErrCtrStopped, "cannot get namespace path unless container %s is running", c.ID())
 	}
@@ -1056,20 +905,73 @@ func (c *Container) NamespacePath(linuxNS LinuxNS) (string, error) { //nolint:in
 	return fmt.Sprintf("/proc/%d/ns/%s", c.state.PID, linuxNS.String()), nil
 }
 
-// CGroupPath returns a cgroups "path" for a given container.
-func (c *Container) CGroupPath() (string, error) {
-	switch c.runtime.config.CgroupManager {
-	case CgroupfsCgroupsManager:
-		return filepath.Join(c.config.CgroupParent, fmt.Sprintf("libpod-%s", c.ID())), nil
-	case SystemdCgroupsManager:
-		if rootless.IsRootless() {
-			uid := rootless.GetRootlessUID()
-			return filepath.Join(c.config.CgroupParent, fmt.Sprintf("user-%d.slice/user@%d.service/user.slice", uid, uid), createUnitName("libpod", c.ID())), nil
-		}
-		return filepath.Join(c.config.CgroupParent, createUnitName("libpod", c.ID())), nil
-	default:
-		return "", errors.Wrapf(define.ErrInvalidArg, "unsupported CGroup manager %s in use", c.runtime.config.CgroupManager)
+// CgroupManager returns the cgroup manager used by the given container.
+func (c *Container) CgroupManager() string {
+	cgroupManager := c.config.CgroupManager
+	if cgroupManager == "" {
+		cgroupManager = c.runtime.config.Engine.CgroupManager
 	}
+	return cgroupManager
+}
+
+// CGroupPath returns a cgroups "path" for the given container.
+// Note that the container must be running.  Otherwise, an error
+// is returned.
+func (c *Container) CGroupPath() (string, error) {
+	if !c.batched {
+		c.lock.Lock()
+		defer c.lock.Unlock()
+		if err := c.syncContainer(); err != nil {
+			return "", errors.Wrapf(err, "error updating container %s state", c.ID())
+		}
+	}
+	return c.cGroupPath()
+}
+
+// cGroupPath returns a cgroups "path" for the given container.
+// Note that the container must be running.  Otherwise, an error
+// is returned.
+// NOTE: only call this when owning the container's lock.
+func (c *Container) cGroupPath() (string, error) {
+	if c.config.NoCgroups || c.config.CgroupsMode == "disabled" {
+		return "", errors.Wrapf(define.ErrNoCgroups, "this container is not creating cgroups")
+	}
+	if c.state.State != define.ContainerStateRunning && c.state.State != define.ContainerStatePaused {
+		return "", errors.Wrapf(define.ErrCtrStopped, "cannot get cgroup path unless container %s is running", c.ID())
+	}
+
+	// Read /proc/{PID}/cgroup and find the *longest* cgroup entry.  That's
+	// needed to account for hacks in cgroups v1, where each line in the
+	// file could potentially point to a cgroup.  The longest one, however,
+	// is the libpod-specific one we're looking for.
+	//
+	// See #8397 on the need for the longest-path look up.
+	procPath := fmt.Sprintf("/proc/%d/cgroup", c.state.PID)
+	lines, err := ioutil.ReadFile(procPath)
+	if err != nil {
+		return "", err
+	}
+
+	var cgroupPath string
+	for _, line := range bytes.Split(lines, []byte("\n")) {
+		// cgroups(7) nails it down to three fields with the 3rd
+		// pointing to the cgroup's path which works both on v1 and v2.
+		fields := bytes.Split(line, []byte(":"))
+		if len(fields) != 3 {
+			logrus.Debugf("Error parsing cgroup: expected 3 fields but got %d: %s", len(fields), procPath)
+			continue
+		}
+		path := string(fields[2])
+		if len(path) > len(cgroupPath) {
+			cgroupPath = path
+		}
+	}
+
+	if len(cgroupPath) == 0 {
+		return "", errors.Errorf("could not find any cgroup in %q", procPath)
+	}
+
+	return cgroupPath, nil
 }
 
 // RootFsSize returns the root FS size of the container
@@ -1103,6 +1005,9 @@ func (c *Container) IDMappings() (storage.IDMappingOptions, error) {
 
 // RootUID returns the root user mapping from container
 func (c *Container) RootUID() int {
+	if len(c.config.IDMappings.UIDMap) == 1 && c.config.IDMappings.UIDMap[0].Size == 1 {
+		return c.config.IDMappings.UIDMap[0].HostID
+	}
 	for _, uidmap := range c.config.IDMappings.UIDMap {
 		if uidmap.ContainerID == 0 {
 			return uidmap.HostID
@@ -1113,6 +1018,9 @@ func (c *Container) RootUID() int {
 
 // RootGID returns the root user mapping from container
 func (c *Container) RootGID() int {
+	if len(c.config.IDMappings.GIDMap) == 1 && c.config.IDMappings.GIDMap[0].Size == 1 {
+		return c.config.IDMappings.GIDMap[0].HostID
+	}
 	for _, gidmap := range c.config.IDMappings.GIDMap {
 		if gidmap.ContainerID == 0 {
 			return gidmap.HostID
@@ -1138,10 +1046,9 @@ func (c *Container) NetworkDisabled() (bool, error) {
 		if err != nil {
 			return false, err
 		}
-		return networkDisabled(container)
+		return container.NetworkDisabled()
 	}
 	return networkDisabled(c)
-
 }
 
 func networkDisabled(c *Container) (bool, error) {
@@ -1158,6 +1065,18 @@ func networkDisabled(c *Container) (bool, error) {
 	return false, nil
 }
 
+func (c *Container) HostNetwork() bool {
+	if c.config.CreateNetNS || c.config.NetNsCtr != "" {
+		return false
+	}
+	for _, ns := range c.config.Spec.Linux.Namespaces {
+		if ns.Type == spec.NetworkNamespace {
+			return false
+		}
+	}
+	return true
+}
+
 // ContainerState returns containerstate struct
 func (c *Container) ContainerState() (*ContainerState, error) {
 	if !c.batched {
@@ -1185,3 +1104,93 @@ func (c *Container) HasHealthCheck() bool {
 func (c *Container) HealthCheckConfig() *manifest.Schema2HealthConfig {
 	return c.config.HealthCheckConfig
 }
+
+// AutoRemove indicates whether the container will be removed after it is executed
+func (c *Container) AutoRemove() bool {
+	spec := c.config.Spec
+	if spec.Annotations == nil {
+		return false
+	}
+	return c.Spec().Annotations[define.InspectAnnotationAutoremove] == define.InspectResponseTrue
+}
+
+// Timezone returns the timezone configured inside the container.
+// Local means it has the same timezone as the host machine
+func (c *Container) Timezone() string {
+	return c.config.Timezone
+}
+
+// Umask returns the Umask bits configured inside the container.
+func (c *Container) Umask() string {
+	return c.config.Umask
+}
+
+//Secrets return the secrets in the container
+func (c *Container) Secrets() []*secrets.Secret {
+	return c.config.Secrets
+}
+
+// Networks gets all the networks this container is connected to.
+// Please do NOT use ctr.config.Networks, as this can be changed from those
+// values at runtime via network connect and disconnect.
+// If the container is configured to use CNI and this function returns an empty
+// array, the container will still be connected to the default network.
+// The second return parameter, a bool, indicates that the container container
+// is joining the default CNI network - the network name will be included in the
+// returned array of network names, but the container did not explicitly join
+// this network.
+func (c *Container) Networks() ([]string, bool, error) {
+	if !c.batched {
+		c.lock.Lock()
+		defer c.lock.Unlock()
+
+		if err := c.syncContainer(); err != nil {
+			return nil, false, err
+		}
+	}
+
+	return c.networks()
+}
+
+// Unlocked accessor for networks
+func (c *Container) networks() ([]string, bool, error) {
+	networks, err := c.runtime.state.GetNetworks(c)
+	if err != nil && errors.Cause(err) == define.ErrNoSuchNetwork {
+		if len(c.config.Networks) == 0 && !rootless.IsRootless() {
+			return []string{c.runtime.netPlugin.GetDefaultNetworkName()}, true, nil
+		}
+		return c.config.Networks, false, nil
+	}
+
+	return networks, false, err
+}
+
+// networksByNameIndex provides us with a map of container networks where key
+// is network name and value is the index position
+func (c *Container) networksByNameIndex() (map[string]int, error) {
+	networks, _, err := c.networks()
+	if err != nil {
+		return nil, err
+	}
+	networkNamesByIndex := make(map[string]int, len(networks))
+	for index, name := range networks {
+		networkNamesByIndex[name] = index
+	}
+	return networkNamesByIndex, nil
+}
+
+// add puts the new given CNI network name into the tracking map
+// and assigns it a new integer based on the map length
+func (d ContainerNetworkDescriptions) add(networkName string) {
+	d[networkName] = len(d)
+}
+
+// getInterfaceByName returns a formatted interface name for a given
+// network along with a bool as to whether the network existed
+func (d ContainerNetworkDescriptions) getInterfaceByName(networkName string) (string, bool) {
+	val, exists := d[networkName]
+	if !exists {
+		return "", exists
+	}
+	return fmt.Sprintf("eth%d", val), exists
+}
